@@ -2,23 +2,25 @@
 RCM_est <- function(x = 1, RCMdata, selectivity, s_selectivity, LWT = list(),
                     comp_like = c("multinomial", "lognormal", "mvlogistic", "dirmult1", "dirmult2"), prior = list(),
                     max_F = 3, integrate = FALSE, StockPars, ObsPars, FleetPars, mean_fit = FALSE,
-                    control = list(iter.max = 2e+05, eval.max = 4e+05), inner.control = list(maxit = 1e3), dots = list()) {
+                    control = list(iter.max = 2e+05, eval.max = 4e+05), inner.control = list(maxit = 1e3), 
+                    start = list(), map = list(), dots = list()) {
   
   comp_like <- match.arg(comp_like)
   
-  if(is.null(dots$rescale)) {
-    if(length(RCMdata@Chist) && any(RCMdata@Chist > 0, na.rm = TRUE)) {
+  if (is.null(dots$rescale)) {
+    if (length(RCMdata@Chist) && any(RCMdata@Chist > 0, na.rm = TRUE)) {
       dots$rescale <- 1/mean(RCMdata@Chist, na.rm = TRUE)
     } else {
       dots$rescale <- 1
     }
   }
   
-  TMB_params <- RCM_est_params(x, RCMdata, selectivity, s_selectivity, prior, LWT, comp_like, StockPars, FleetPars, dots)
+  TMB_params <- RCM_est_params(x, RCMdata, selectivity, s_selectivity, prior, LWT, comp_like, StockPars, FleetPars, 
+                               start, map, dots)
   TMB_data <- RCM_est_data(x, RCMdata, selectivity, s_selectivity, LWT, comp_like, prior, max_F, 
                            StockPars, ObsPars, FleetPars, mean_fit, TMB_params$map, dots)
   
-  if(integrate) {
+  if (integrate) {
     random <- c("log_early_rec_dev", "log_rec_dev") 
   } else {
     random <- NULL
@@ -27,16 +29,16 @@ RCM_est <- function(x = 1, RCMdata, selectivity, s_selectivity, LWT = list(),
   obj <- MakeADFun(data = TMB_data, parameters = TMB_params$params, map = TMB_params$map, random = random,
                    inner.control = inner.control, DLL = "SAMtool", silent = TRUE)
   
-  if(RCMdata@Misc$condition == "catch2") {
-    if(any(is.na(obj$report(obj$par)$F)) || any(is.infinite(obj$report(obj$par)$F))) {
+  if (RCMdata@Misc$condition == "catch2") {
+    if (any(is.na(obj$report(obj$par)$F)) || any(is.infinite(obj$report(obj$par)$F))) {
       for(i in 1:10) {
         obj$par["R0x"] <- 0.5 + obj$par["R0x"]
-        if(all(!is.na(obj$report(obj$par)$F)) && all(!is.infinite(obj$report(obj$par)$F))) break
+        if (all(!is.na(obj$report(obj$par)$F)) && all(!is.infinite(obj$report(obj$par)$F))) break
       }
     }
   }
   
-  mod <- optimize_TMB_model(obj, control, restart = 0)
+  mod <- optimize_TMB_model(obj, control)
   opt <- mod[[1]]
   SD <- mod[[2]]
   report <- obj$report(obj$env$last.par.best) %>% RCM_posthoc_adjust(obj)
@@ -49,7 +51,11 @@ RCM_est <- function(x = 1, RCMdata, selectivity, s_selectivity, LWT = list(),
 RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), comp_like, prior, max_F,
                          StockPars, ObsPars, FleetPars, mean_fit = FALSE, map, dots = list()) {
   
-  SR_type <- ifelse(StockPars$SRrel[x] == 1, "BH", "Ricker")
+  SR_type <- switch(StockPars$SRrel[x],
+                    "1" = "BH",
+                    "2" = "Ricker",
+                    "3" = "Mesnil-Rochet")
+  if (is.null(SR_type)) stop("Can not identify stock-recruit function in OM@SRrel")
   
   nyears <- RCMdata@Misc$nyears
   nfleet <- RCMdata@Misc$nfleet
@@ -63,7 +69,7 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
   RCMdata@IAA <- apply(RCMdata@IAA, c(1, 3), tiny_comp) %>% aperm(c(2, 1, 3))
   RCMdata@IAL <- apply(RCMdata@IAL, c(1, 3), tiny_comp) %>% aperm(c(2, 1, 3))
   
-  if(mean_fit) {
+  if (mean_fit) {
     # Average across simulations for arrays: M_ageArray, Len_age, Mat_age (mean across index 1)
     # Matrices: L5, LFS, Vmaxlen (mean across rows)
     # Vectors: Isd, Len_CV, hs, R0 (effort only)
@@ -84,9 +90,9 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
     StockPars_ind <- match("ageMarray", names(StockPars))
     StockPars[StockPars_ind] <- lapply(StockPars[StockPars_ind], mean_matrix)
     
-    if(RCMdata@Misc$condition == "effort") {
+    if (RCMdata@Misc$condition == "effort") {
       StockPars$R0 <- mean_vector(StockPars$R0)
-      if(!is.null(dots$OMeff) && dots$OMeff) {
+      if (!is.null(dots$OMeff) && dots$OMeff) {
         FleetPars$Find <- apply(FleetPars$Find, 2, mean) %>% matrix(length(StockPars$R0), nyears, byrow = TRUE)
       }
     }
@@ -97,21 +103,21 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
     ObsPars$Isd <- mean_vector(ObsPars$Isd)
   }
   
-  if(!length(RCMdata@I_sd) || !any(RCMdata@I_sd > 0, na.rm = TRUE)) {
+  if (!length(RCMdata@I_sd) || !any(RCMdata@I_sd > 0, na.rm = TRUE)) {
     RCMdata@I_sd <- matrix(sdconv(1, ObsPars$Isd[x]), nyears, nsurvey)
   }
   
-  if(length(RCMdata@Chist) && any(RCMdata@Chist > 0, na.rm = TRUE)) {
+  if (length(RCMdata@Chist) && any(RCMdata@Chist > 0, na.rm = TRUE)) {
     C_hist <- RCMdata@Chist
   } else {
     C_hist <- matrix(0, nyears, nfleet)
   }
   
-  if(RCMdata@Misc$condition == "effort" && !is.null(dots$OMeff) && dots$OMeff) {
+  if (RCMdata@Misc$condition == "effort" && !is.null(dots$OMeff) && dots$OMeff) {
     RCMdata@Ehist <- matrix(FleetPars$Find[x, ], nyears, nfleet)
   }
   
-  if(length(RCMdata@Ehist) && any(RCMdata@Ehist > 0, na.rm = TRUE)) {
+  if (length(RCMdata@Ehist) && any(RCMdata@Ehist > 0, na.rm = TRUE)) {
     rescale_effort <- 1/mean(RCMdata@Ehist, na.rm = TRUE)
     E_hist <- RCMdata@Ehist * rescale_effort
     E_eq <- RCMdata@E_eq * rescale_effort
@@ -125,8 +131,8 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
   LWT_fleet <- cbind(LWT$Chist, LWT$C_eq, LWT$CAA, LWT$CAL, LWT$MS)
   LWT_index <- cbind(LWT$Index, LWT$IAA, LWT$IAL)
   
-  if(is.null(dots$n_itF)) n_itF <- 3L else n_itF <- dots$n_itF
-  if(is.null(dots$plusgroup)) plusgroup <- 1L else plusgroup <- as.integer(dots$plusgroup)
+  if (is.null(dots$n_itF)) n_itF <- 3L else n_itF <- dots$n_itF
+  if (is.null(dots$plusgroup)) plusgroup <- 1L else plusgroup <- as.integer(dots$plusgroup)
   age_only_model <- StockPars$Len_age[x, , 1:(nyears+1)] %>%
     apply(2, function(xx) all(xx == 1:n_age)) %>% 
     all()
@@ -145,7 +151,7 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
                    sel_block = rbind(RCMdata@sel_block, RCMdata@sel_block[nyears, ]), 
                    nsel_block = RCMdata@Misc$nsel_block,
                    n_y = nyears, n_age = n_age, nfleet = nfleet, nsurvey = nsurvey,
-                   M_data = if(prior$use_prior[3]) matrix(1, 1, 1) else t(StockPars$M_ageArray[x, , 1:nyears]), 
+                   M_data = if (prior$use_prior[3]) matrix(1, 1, 1) else t(StockPars$M_ageArray[x, , 1:nyears]), 
                    len_age = t(StockPars$Len_age[x, , 1:(nyears+1)]),
                    Linf = ifelse(age_only_model, n_age, StockPars$Linf[x]),
                    SD_LAA = t(StockPars$LatASD[x, , 1:nyears]), 
@@ -166,7 +172,8 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
                    n_itF = n_itF, plusgroup = plusgroup,
                    use_prior = prior$use_prior, 
                    prior_dist = prior$pr_matrix, 
-                   nll_gr = 0L)
+                   nll_gr = 0L,
+                   sim_process_error = 0L)
                    
   TMB_data$est_vul <- ifelse(is.na(map$vul_par) | duplicated(map$vul_par), 0, 1) %>%
     matrix(length(map$vul_par)/RCMdata@Misc$nsel_block, RCMdata@Misc$nsel_block)
@@ -185,38 +192,49 @@ RCM_est_data <- function(x, RCMdata, selectivity, s_selectivity, LWT = list(), c
 }
 
 RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(), LWT = list(), comp_like,
-                           StockPars, FleetPars, dots = list()) {
+                           StockPars, FleetPars, start = list(), map = list(), dots = list()) {
   
-  SR_type <- ifelse(StockPars$SRrel[x] == 1, "BH", "Ricker")
+  if (!is.null(dots$vul_par)) stop("Specify vul_par via start$vul_par")
+  if (!is.null(dots$ivul_par)) stop("Specify ivul_par via start$vul_par")
+  if (!is.null(dots$log_rec_dev)) stop("Specify log_rec_dev via start$log_rec_dev")
+  if (!is.null(dots$log_early_rec_dev)) stop("Specify log_early_rec_dev via start$log_early_rec_dev")
+  
+  if (!is.null(dots$map_vul_par)) stop("Specify map_vul_par via map$vul_par")
+  if (!is.null(dots$map_ivul_par)) stop("Specify map_ivul_par via map$ivul_par")
+  if (!is.null(dots$map_log_rec_dev)) stop("Specify map_log_rec_dev via map$log_rec_dev")
+  if (!is.null(dots$map_log_early_rec_dev)) stop("Specify map_log_early_rec_dev via map$log_early_rec_dev")
+  
+  SR_type <- switch(StockPars$SRrel[x],
+                    "1" = "BH",
+                    "2" = "Ricker",
+                    "3" = "Mesnil-Rochet")
+  if (is.null(SR_type)) stop("Can not identify stock-recruit function in OM@SRrel")
   
   nyears <- RCMdata@Misc$nyears
   nfleet <- RCMdata@Misc$nfleet
   n_age <- dim(RCMdata@CAA)[2]
   nsurvey <- ncol(RCMdata@Index)
   
-  if(SR_type == "BH") {
-    transformed_h <- logit((StockPars$hs[x] - 0.2)/0.8)
-  } else transformed_h <- log(StockPars$hs[x] - 0.2)
+  transformed_h <- switch(SR_type,
+                          "BH" = logit((StockPars$hs[x] - 0.2)/0.8),
+                          "Ricker" = log(StockPars$hs[x] - 0.2),
+                          "Mesnil-Rochet" = 0)
   
   LFS <- FleetPars$LFS_y[x, nyears]
   L5 <- FleetPars$L5_y[x, nyears]
   Vmaxlen <- FleetPars$Vmaxlen_y[x, nyears]
   
-  if(is.null(dots$vul_par)) {
-    if(any(selectivity == -2)) {
-      stop("Some fleet selectivity specified to be free parameters. Provide vul_par matrix to RCM.")
+  if (is.null(start$vul_par)) {
+    if (any(selectivity == -2)) {
+      stop("Some fleet selectivity specified to be free parameters. Provide start$vul_par matrix to RCM.")
     }
-    vul_par <- matrix(c(LFS, L5, Vmaxlen), 3, RCMdata@Misc$nsel_block)
-  } else {
-    vul_par <- dots$vul_par
-    
-    if(ncol(vul_par) != RCMdata@Misc$nsel_block) {
-      stop("vul_par needs to be a matrix with ", RCMdata@Misc$nsel_block, " columns")
-    }
-    if(any(selectivity == -2) && nrow(vul_par) != n_age) {
-      stop("vul_par needs to be a matrix with ", n_age, " (maxage + 1) rows")
-    }
-    
+    start$vul_par <- matrix(c(LFS, L5, Vmaxlen), 3, RCMdata@Misc$nsel_block)
+  }
+  if (ncol(start$vul_par) != RCMdata@Misc$nsel_block) {
+    stop("start$vul_par needs to be a matrix with ", RCMdata@Misc$nsel_block, " columns")
+  }
+  if (any(selectivity == -2) && nrow(start$vul_par) != n_age) {
+    stop("start$vul_par needs to be a matrix with ", n_age, " (maxage + 1) rows")
   }
   
   # Check for functional selectivity functions (dome or logistic)
@@ -226,155 +244,157 @@ RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(
   Linf <- ifelse(age_only_model, n_age, StockPars$Linf[x])
   
   sel_check <- selectivity == -1 | selectivity == 0
-  vul_par[2, sel_check] <- log(vul_par[1, sel_check] - vul_par[2, sel_check])
-  vul_par[1, sel_check] <- logit(pmin(vul_par[1, sel_check]/Linf/0.99, 0.99))
-  vul_par[3, sel_check] <- logit(pmin(vul_par[3, sel_check], 0.99))
+  start$vul_par[2, sel_check] <- log(start$vul_par[1, sel_check] - start$vul_par[2, sel_check])
+  start$vul_par[1, sel_check] <- logit(pmin(start$vul_par[1, sel_check]/Linf/0.99, 0.99))
+  start$vul_par[3, sel_check] <- logit(pmin(start$vul_par[3, sel_check], 0.99))
   
-  if(any(selectivity == -2)) {
-    vul_par[, selectivity == -2] <- logit(vul_par[, selectivity == -2], soft_bounds = TRUE)
+  if (any(selectivity == -2)) {
+    start$vul_par[, selectivity == -2] <- logit(start$vul_par[, selectivity == -2], soft_bounds = TRUE)
   }
   
-  if(is.null(dots$map_vul_par)) {
-    if(any(selectivity == -2)) {
+  if (is.null(map$vul_par)) {
+    if (any(selectivity == -2)) {
       stop("Some (fleet) selectivity specified to be free parameters. Provide map_vul_par matrix to RCM.")
     }
-    map_vul_par <- matrix(0, 3, RCMdata@Misc$nsel_block)
-    map_vul_par[3, selectivity == -1] <- NA # Fix third parameter for logistic sel
-    if(!is.null(dots$fix_dome) && dots$fix_dome) { # Obsolete
-      map_vul_par[3, selectivity == 0] <- NA # Fix dome
+    map$vul_par <- matrix(0, 3, RCMdata@Misc$nsel_block)
+    map$vul_par[3, selectivity == -1] <- NA # Fix third parameter for logistic sel
+    if (!is.null(dots$fix_dome) && dots$fix_dome) { # Obsolete
+      map$vul_par[3, selectivity == 0] <- NA # Fix dome
     }
     
     for(ff in 1:nfleet) {
-      if(all(RCMdata@CAA[,,ff] <= 0, na.rm = TRUE) && all(RCMdata@CAL[,,ff] <= 0, na.rm = TRUE)) {
-        map_vul_par[, unique(RCMdata@sel_block[, ff])] <- NA # Fix sel if no comp data
+      if (all(RCMdata@CAA[,,ff] <= 0, na.rm = TRUE) && all(RCMdata@CAL[,,ff] <= 0, na.rm = TRUE)) {
+        map$vul_par[, unique(RCMdata@sel_block[, ff])] <- NA # Fix sel if no comp data
       }
     }
-    if(any(!is.na(map_vul_par))) {
-      map_vul_par[!is.na(map_vul_par)] <- 1:sum(!is.na(map_vul_par))
-    }
-  } else {
-    map_vul_par <- dots$map_vul_par
-    
-    if(ncol(map_vul_par) != RCMdata@Misc$nsel_block) {
-      stop("map_vul_par needs to be a matrix with ", RCMdata@Misc$nsel_block, " columns")
-    }
-    if(any(selectivity == -2) && nrow(map_vul_par) != n_age) {
-      stop("map_vul_par needs to be a matrix with ", n_age, " (maxage + 1) rows")
+    if (any(!is.na(map$vul_par))) {
+      map$vul_par[!is.na(map$vul_par)] <- 1:sum(!is.na(map$vul_par))
     }
   }
+  if (ncol(map$vul_par) != RCMdata@Misc$nsel_block) {
+    stop("map$vul_par needs to be a matrix with ", RCMdata@Misc$nsel_block, " columns")
+  }
+  if (any(selectivity == -2) && nrow(map$vul_par) != n_age) {
+    stop("map$vul_par needs to be a matrix with ", n_age, " (maxage + 1) rows")
+  }
   
-  if(any(selectivity == -2)) {
-    test <- dots$vul_par[, selectivity == -2] %in% c(0, 1) & is.na(map_vul_par[, selectivity == -2])
-    vul_par[, selectivity == -2][test] <- logit(dots$vul_par[, selectivity == -2][test], soft_bounds = FALSE)
+  if (any(selectivity == -2)) {
+    test <- start$vul_par[, selectivity == -2] %in% c(0, 1) & is.na(map$vul_par[, selectivity == -2])
+    start$vul_par[, selectivity == -2][test] <- logit(start$vul_par[, selectivity == -2][test], soft_bounds = FALSE)
   }
   
   # Index selectivity (ivul_par)
-  if(!is.null(dots$s_vul_par)) dots$ivul_par <- dots$s_vul_par # Backwards compatibility
-  if(is.null(dots$ivul_par)) {
-    if(any(s_selectivity == -2)) {
-      stop("Some s_selectivity specified to be free parameters. Provide ivul_par matrix to RCM.")
-    }
-    ivul_par <- matrix(c(LFS, L5, Vmaxlen), 3, nsurvey)
-  } else {
-    ivul_par <- dots$ivul_par
-    
-    if(ncol(ivul_par) != nsurvey) {
-      stop("ivul_par needs to be a matrix with ", nsurvey, " columns.")
-    }
-    if(any(s_selectivity == -2) && nrow(ivul_par) != n_age) {
-      stop("ivul_par needs to be a matrix with ", n_age, " (maxage + 1) rows.")
+  if (!is.null(dots$s_vul_par)) stop("Pass index selectivity parameters via start$ivul_par")
+  if (is.null(start$ivul_par)) {
+    if (any(s_selectivity == -2)) {
+      stop("Some s_selectivity specified to be free parameters. Provide start$ivul_par matrix to RCM.")
     }
+    start$ivul_par <- matrix(c(LFS, L5, Vmaxlen), 3, nsurvey)
+  }
+  if (ncol(start$ivul_par) != nsurvey) {
+    stop("start$ivul_par needs to be a matrix with ", nsurvey, " columns.")
+  }
+  if (any(s_selectivity == -2) && nrow(start$ivul_par) != n_age) {
+    stop("start$ivul_par needs to be a matrix with ", n_age, " (maxage + 1) rows.")
   }
   
   # Check for function selectivity (dome/logistic)
   parametric_sel <- s_selectivity == -1 | s_selectivity == 0
-  ivul_par[2, parametric_sel] <- log(ivul_par[1, parametric_sel] - ivul_par[2, parametric_sel])
-  ivul_par[1, parametric_sel] <- logit(ivul_par[1, parametric_sel]/Linf/0.99)
-  ivul_par[3, parametric_sel] <- logit(ivul_par[3, parametric_sel])
+  start$ivul_par[2, parametric_sel] <- log(start$ivul_par[1, parametric_sel] - start$ivul_par[2, parametric_sel])
+  start$ivul_par[1, parametric_sel] <- logit(start$ivul_par[1, parametric_sel]/Linf/0.99)
+  start$ivul_par[3, parametric_sel] <- logit(start$ivul_par[3, parametric_sel])
   
-  if(any(s_selectivity == -2)) {
-    ivul_par[, s_selectivity == -2] <- logit(ivul_par[, s_selectivity == -2], soft_bounds = TRUE)
+  if (any(s_selectivity == -2)) {
+    start$ivul_par[, s_selectivity == -2] <- logit(start$ivul_par[, s_selectivity == -2], soft_bounds = TRUE)
   }
-  if(!is.null(dots$map_s_vul_par)) dots$map_ivul_par <- dots$map_s_vul_par # Backwards compatibility
   
-  if(is.null(dots$map_ivul_par)) {
-    if(any(s_selectivity == -2)) {
+  if (!is.null(dots$map_s_vul_par)) stop("Pass map argument of index selectivity parameters via map$ivul_par")
+  if (is.null(map$ivul_par)) {
+    if (any(s_selectivity == -2)) {
       stop("Some s_selectivity specified to be free parameters. Provide map_ivul_par matrix to RCM.")
     }
-    map_ivul_par <- matrix(0, 3, nsurvey)
-    map_ivul_par[3, s_selectivity < 0] <- NA # if logistic
+    map$ivul_par <- matrix(0, 3, nsurvey)
+    map$ivul_par[3, s_selectivity < 0] <- NA # if logistic
     for(sur in 1:nsurvey) {
-      if(s_selectivity[sur] < -2 || s_selectivity[sur] > 0 ||
+      if (s_selectivity[sur] < -2 || s_selectivity[sur] > 0 ||
          (all(RCMdata@IAA[,,sur] <= 0, na.rm = TRUE) & all(RCMdata@IAL[,,sur] <= 0, na.rm = TRUE))) {
-        map_ivul_par[, sur] <- NA
+        map$ivul_par[, sur] <- NA
       }
     }
-    if(any(!is.na(map_ivul_par))) {
-      map_ivul_par[!is.na(map_ivul_par)] <- 1:sum(!is.na(map_ivul_par))
-    }
-  } else {
-    map_ivul_par <- dots$map_ivul_par
-    
-    if(ncol(map_ivul_par) != nsurvey) {
-      stop("ivul_par needs to be a matrix with ", nsurvey, " columns.")
-    }
-    if(any(s_selectivity == -2) && nrow(map_ivul_par) != n_age) {
-      stop("ivul_par needs to be a matrix with ", n_age, " (maxage + 1) rows.")
+    if (any(!is.na(map$ivul_par))) {
+      map$ivul_par[!is.na(map$ivul_par)] <- 1:sum(!is.na(map$ivul_par))
     }
   }
-  
-  if(any(s_selectivity == -2)) {
-    test <- dots$ivul_par[, s_selectivity == -2] %in% c(0, 1) & is.na(map_ivul_par[, s_selectivity == -2])
-    ivul_par[, s_selectivity == -2][test] <- logit(dots$ivul_par[, s_selectivity == -2][test], soft_bounds = FALSE)
+  if (ncol(map$ivul_par) != nsurvey) {
+    stop("map$ivul_par needs to be a matrix with ", nsurvey, " columns.")
+  }
+  if (any(s_selectivity == -2) && nrow(map$ivul_par) != n_age) {
+    stop("map$ivul_par needs to be a matrix with ", n_age, " (maxage + 1) rows.")
   }
   
-  if(!is.null(dots$log_early_rec_dev)) {
-    if(length(dots$log_early_rec_dev) != n_age - 1) stop("early_rec_dev is not a vector of length n_age - 1")
-    log_early_rec_dev <- dots$log_early_rec_dev
-  } else {
-    log_early_rec_dev <- rep(0, n_age - 1)
+  if (any(s_selectivity == -2)) {
+    test <- start$ivul_par[, s_selectivity == -2] %in% c(0, 1) & is.na(map$ivul_par[, s_selectivity == -2])
+    start$ivul_par[, s_selectivity == -2][test] <- logit(start$ivul_par[, s_selectivity == -2][test], soft_bounds = FALSE)
   }
   
-  if(!is.null(dots$log_rec_dev)) {
-    if(length(dots$log_rec_dev) != nyears) stop("log_rec_dev is not a vector of length nyears")
-    log_rec_dev <- dots$log_rec_dev
-  } else {
-    log_rec_dev <- rep(0, nyears)
-  }
+  if (is.null(start$log_early_rec_dev)) start$log_early_rec_dev <- rep(0, n_age - 1)
+  if (length(start$log_early_rec_dev) != n_age - 1) stop("start$log_early_rec_dev is not a vector of length n_age - 1")
+  
+  if (is.null(start$log_rec_dev)) start$log_rec_dev <- rep(0, nyears)
+  if (length(start$log_rec_dev) != nyears) stop("start$log_rec_dev is not a vector of length nyears")
+  
+  # Mesnil Rochet parameters
+  if (is.null(start$MR_SRR)) start$MR_SRR <- c(0.8, 0.001)
+  if (length(start$MR_SRR) != 2) stop("start$MR_SRR needs to be a length 2 vector.")
   
   TMB_params <- list(R0x = ifelse(!is.na(StockPars$R0[x]), log(StockPars$R0[x] * dots$rescale), 0),
-                     transformed_h = transformed_h, log_M = log(mean(StockPars$M_ageArray[x, , nyears])),
-                     vul_par = vul_par, ivul_par = ivul_par,
-                     log_q_effort = rep(log(0.1), nfleet), log_F_dev = matrix(0, nyears, nfleet),
+                     transformed_h = transformed_h, 
+                     MR_SRR = c(logit(start$MR_SRR[1]), log(start$MR_SRR[2])), # Shinge/S0, gamma 
+                     log_M = log(mean(StockPars$M_ageArray[x, , nyears])),
+                     vul_par = start$vul_par, 
+                     ivul_par = start$ivul_par,
+                     log_q_effort = rep(log(0.1), nfleet), 
+                     log_F_dev = matrix(0, nyears, nfleet),
                      log_F_equilibrium = rep(log(0.05), nfleet),
-                     log_CV_msize = log(RCMdata@MS_cv), log_tau = log(StockPars$procsd[x]),
-                     log_early_rec_dev = log_early_rec_dev, log_rec_dev = log_rec_dev,
-                     log_compf = matrix(0, nfleet, 2), log_compi = matrix(0, nsurvey, 2))
+                     log_CV_msize = log(RCMdata@MS_cv), 
+                     log_tau = log(StockPars$procsd[x]),
+                     log_early_rec_dev = start$log_early_rec_dev, 
+                     log_rec_dev = start$log_rec_dev,
+                     log_compf = matrix(0, nfleet, 2),
+                     log_compi = matrix(0, nsurvey, 2))
   
-  if(RCMdata@Misc$condition == "catch") {
+  if (RCMdata@Misc$condition == "catch") {
     TMB_params$log_F_dev[as.integer(0.5 * nyears) + 1, ] <- log(0.5 * mean(StockPars$M_ageArray[x, , nyears]))
   }
   
   # Map list (to fix parameters)
-  map <- list()
+  map_out <- list()
   
-  if(RCMdata@Misc$condition == "effort" && !sum(RCMdata@Chist, na.rm = TRUE) && !prior$use_prior[1]) {
-    map$R0x <- factor(NA) # Fix if condition on effort, no catches, and no prior on R0
+  if (RCMdata@Misc$condition == "effort" && !sum(RCMdata@Chist, na.rm = TRUE) && !prior$use_prior[1]) {
+    map_out$R0x <- factor(NA) # Fix if condition on effort, no catches, and no prior on R0
   }
-  if(!prior$use_prior[2]) map$transformed_h <- factor(NA)
-  if(!prior$use_prior[3]) map$log_M <- factor(NA)
+  if (SR_type == "Mesnil-Rochet" || !prior$use_prior[2]) map_out$transformed_h <- factor(NA)
   
-  map$log_tau <- factor(NA)
-  map$vul_par <- factor(map_vul_par)
-  map$ivul_par <- factor(map_ivul_par)
-  
-  if(RCMdata@Misc$condition == "effort") {
-    map$log_F_equilibrium <- factor(rep(NA, nfleet))
+  if (SR_type == "Mesnil-Rochet") {
+    if (is.null(map$MR_SRR)) map$MR_SRR <- c(1, NA)  # Estimates Ehinge/E0, fixes gamma = 0.001 --> hockey-stick
+    map_out$MR_SRR <- factor(map$MR_SRR)
   } else {
-    map$log_q_effort <- factor(rep(NA, nfleet))
+    map_out$MR_SRR <- factor(c(NA, NA))
+  }
+  
+  if (!prior$use_prior[3]) map_out$log_M <- factor(NA)
+  
+  map_out$log_tau <- factor(NA)
+  map_out$vul_par <- factor(map$vul_par)
+  map_out$ivul_par <- factor(map$ivul_par)
+  
+  if (RCMdata@Misc$condition == "effort") {
+    map_out$log_F_equilibrium <- factor(rep(NA, nfleet))
+  } else {
+    map_out$log_q_effort <- factor(rep(NA, nfleet))
     
-    if(any(RCMdata@C_eq == 0)) {
-      map$log_F_equilibrium <- local({
+    if (any(RCMdata@C_eq == 0)) {
+      map_out$log_F_equilibrium <- local({
         m <- rep(NA, nfleet)
         m[RCMdata@C_eq > 0] <- 1:sum(RCMdata@C_eq > 0)
         factor(m)
@@ -382,33 +402,33 @@ RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(
     }
   }
   
-  if(RCMdata@Misc$condition != "catch") {
-    map$log_F_dev <- factor(matrix(NA, nyears, nfleet))
+  if (RCMdata@Misc$condition != "catch") {
+    map_out$log_F_dev <- factor(matrix(NA, nyears, nfleet))
   }
-  map$log_CV_msize <- factor(rep(NA, nfleet))
+  map_out$log_CV_msize <- factor(rep(NA, nfleet))
   
-  if(is.null(dots$map_log_early_rec_dev)) {
-    map$log_early_rec_dev <- factor(rep(NA, n_age - 1))
+  if (is.null(map$log_early_rec_dev)) {
+    map_out$log_early_rec_dev <- factor(rep(NA, n_age - 1))
   } else {
-    map$log_early_rec_dev <- factor(dots$map_log_early_rec_dev)
-    if(length(map$log_early_rec_dev) != n_age - 1) {
-      stop("map_log_early_rec_dev needs to be a vector of length ", n_age - 1)
+    map_out$log_early_rec_dev <- factor(map$log_early_rec_dev)
+    if (length(map_out$log_early_rec_dev) != n_age - 1) {
+      stop("map$log_early_rec_dev needs to be a vector of length ", n_age - 1)
     }
   }
   
-  if(is.null(dots$map_log_rec_dev)) {
-    map$log_rec_dev <- factor(1:nyears)
+  if (is.null(map$log_rec_dev)) {
+    map_out$log_rec_dev <- factor(1:nyears)
   } else {
-    map$log_rec_dev <- factor(dots$map_log_rec_dev)
-    if(length(map$log_rec_dev) != nyears) {
-      stop("map_log_rec_dev needs to be a vector of length ", nyears)
+    map_out$log_rec_dev <- factor(map$log_rec_dev)
+    if (length(map_out$log_rec_dev) != nyears) {
+      stop("map$log_rec_dev needs to be a vector of length ", nyears)
     }
   }
   
-  map$log_compf <- local({
+  map_out$log_compf <- local({
     mapf <- matrix(NA, nfleet, 2)
     
-    if(comp_like %in% c("dirmult1", "dirmult2")) {
+    if (comp_like %in% c("dirmult1", "dirmult2")) {
       CAAv <- sapply(1:nfleet, function(ff) LWT$CAA[ff] > 0 && sum(RCMdata@CAA_ESS[, ff]) > 0)
       CALv <- sapply(1:nfleet, function(ff) LWT$CAL[ff] > 0 && sum(RCMdata@CAL_ESS[, ff]) > 0)
       
@@ -419,10 +439,10 @@ RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(
     factor(mapf)
   })
   
-  map$log_compi <- local({
+  map_out$log_compi <- local({
     mapi <- matrix(NA, nsurvey, 2)
     
-    if(comp_like %in% c("dirmult1", "dirmult2")) {
+    if (comp_like %in% c("dirmult1", "dirmult2")) {
       IAAv <- sapply(1:nsurvey, function(sur) LWT$IAA[sur] > 0 && sum(RCMdata@IAA_ESS[, sur]) > 0)
       IALv <- sapply(1:nsurvey, function(sur) LWT$IAL[sur] > 0 && sum(RCMdata@IAL_ESS[, sur]) > 0)
       
@@ -432,7 +452,7 @@ RCM_est_params <- function(x, RCMdata, selectivity, s_selectivity, prior = list(
     factor(mapi)
   })
   
-  list(params = TMB_params, map = map)
+  list(params = TMB_params, map = map_out)
 }
 
 par_identical_sims_fn <- function(StockPars, FleetPars, ObsPars, RCMdata, dots) {
@@ -441,28 +461,28 @@ par_identical_sims_fn <- function(StockPars, FleetPars, ObsPars, RCMdata, dots)
     x_mean <- apply(x, 2:length(dim(x)), mean)
     all(apply(x, 1, identical, x_mean))
   }
-  run_test <- function(x) if(is.null(dim(x))) vector_fn(x) else array_fn(x)
+  run_test <- function(x) if (is.null(dim(x))) vector_fn(x) else array_fn(x)
   
   StockPars_subset <- StockPars[c("hs", "procsd", "ageMarray", "M_ageArray", "Linf", "Len_age", "Wt_age", "Mat_age")]
-  if(any(RCMdata@CAL > 0, na.rm = TRUE) || any(RCMdata@IAL > 0, na.rm = TRUE) || 
+  if (any(RCMdata@CAL > 0, na.rm = TRUE) || any(RCMdata@IAL > 0, na.rm = TRUE) || 
      (RCMdata@MS_type == "length" & any(RCMdata@MS > 0, na.rm = TRUE))) {
     StockPars_subset <- c(StockPars_subset, StockPars["LenCV"])
   }
   S_test <- vapply(StockPars_subset, run_test, logical(1))
   
-  if(RCMdata@Misc$nfleet == 1 && !any(RCMdata@CAL > 0, na.rm = TRUE) && !any(RCMdata@CAA > 0, na.rm = TRUE)) {
+  if (RCMdata@Misc$nfleet == 1 && !any(RCMdata@CAL > 0, na.rm = TRUE) && !any(RCMdata@CAA > 0, na.rm = TRUE)) {
     FleetPars_subset <- FleetPars[c("L5_y", "LFS_y", "Vmaxlen_y")]
     FleetPars_subset <- lapply(FleetPars_subset, function(x) x[, RCMdata@Misc$nyears])
     F_test_sel <- vapply(FleetPars_subset, run_test, logical(1))
   } else F_test_sel <- NULL
   
-  if(RCMdata@Misc$condition == "effort" && !is.null(dots$OMeff) && dots$OMeff) {
+  if (RCMdata@Misc$condition == "effort" && !is.null(dots$OMeff) && dots$OMeff) {
     F_test_Find <- run_test(FleetPars$Find) %>% structure(names = "Find")
   } else F_test_Find <- NULL
   
   F_test <- c(F_test_sel, F_test_Find)
   
-  if(RCMdata@Misc$nsurvey > 0 && !any(RCMdata@I_sd > 0, na.rm = TRUE)) {
+  if (RCMdata@Misc$nsurvey > 0 && !any(RCMdata@I_sd > 0, na.rm = TRUE)) {
     O_test <- run_test(ObsPars$Isd) %>% structure(names = "Isd")
   } else {
     O_test <- NULL
@@ -475,19 +495,18 @@ par_identical_sims_fn <- function(StockPars, FleetPars, ObsPars, RCMdata, dots)
 
 RCM_dynamic_SSB0 <- function(obj, par = obj$env$last.par.best) {
   
-  if(obj$env$data$condition == "catch") {
+  if (obj$env$data$condition == "catch") {
     
     par[names(par) == "log_F_dev" | names(par) == "log_F_equilibrium"] <- log(1e-8)
     out <- obj$report(par)$E
     
-  } else if(obj$env$data$condition == "catch2") {
+  } else if (obj$env$data$condition == "catch2") {
     
-    new_args <- RCM_retro_subset(obj$env$data$n_y, data = obj$env$data, 
-                                 params = obj$env$parameters, map = obj$env$map)
-    new_args$data$C_hist <- matrix(1e-8, new_args$data$n_y, new_args$data$nfleet)
+    new_data <- obj$env$data
+    new_data$C_hist[] <- 1e-8
     
-    obj2 <- MakeADFun(data = new_args$data, parameters = new_args$params, 
-                      map = new_args$map, random = obj$env$random,
+    obj2 <- MakeADFun(data = new_data, parameters = clean_tmb_parameters(obj), 
+                      map = obj$env$map, random = obj$env$random,
                       DLL = "SAMtool", silent = TRUE)
     out <- obj2$report(par)$E
     
@@ -507,7 +526,7 @@ RCM_dynamic_SSB0 <- function(obj, par = obj$env$last.par.best) {
 # selectivity at length
 RCM_posthoc_adjust <- function(report, obj, par = obj$env$last.par.best, dynamic_SSB0 = TRUE) {
   data <- obj$env$data
-  if(data$use_prior[3]) {
+  if (data$use_prior[3]) {
     M <- matrix(report$Mest, data$n_y, data$n_age)
   } else {
     M <- data$M_data
@@ -520,18 +539,35 @@ RCM_posthoc_adjust <- function(report, obj, par = obj$env$last.par.best, dynamic
                             equilibrium = FALSE)
   
   report$ageM <- data$ageM
-  if(data$SR_type == "BH") {
+  if (data$SR_type == "BH") {
     report$E0 <- pmax((report$Arec * report$EPR0 - 1)/report$Brec, 0)
     report$h_annual <- local({
       h <- report$Arec * report$EPR0/(4 + report$Arec * report$EPR0)
       ifelse(h < 0.2, NA_real_, h)
     })
-  } else {
+    
+    report$CR <- report$Arec * report$EPR0 # Annual compensation ratio recalculated from annual EPR0
+  } else if (data$SR_type == "Ricker") {
     report$E0 <- pmax(log(report$Arec * report$EPR0)/report$Brec, 0)
     report$h_annual <- local({
       h <- 0.2 * (report$Arec * report$EPR0)^0.8
       ifelse(h < 0.2, NA_real_, h)
     })
+    
+    report$CR <- report$Arec * report$EPR0 # Annual compensation ratio recalculated from annual EPR0
+  } else if (data$SR_type == "Mesnil-Rochet") {
+    
+    MR_K <- sqrt(report$MRhinge^2 + 0.25 * report$MRgamma^2)
+    MR_beta <- report$MRRmax/(report$MRhinge + MR_K)
+    
+    report$E0 <- local({
+      num <- 2 * MR_K / report$EPR0 / MR_beta - 2 * (report$MRhinge + MR_K)
+      den <- 1/report$EPR0/report$EPR0/MR_beta/MR_beta - 2/report$EPR0/MR_beta
+      
+      ifelse(1/report$EPR0 > 2 * MR_beta, 0, num/den)
+    })
+    
+    report$CR <- 2 * MR_beta * report$EPR0 # Annual compensation ratio recalculated from annual EPR0
   }
   report$R0_annual <- report$E0/report$EPR0
   report$N0 <- apply(report$NPR_unfished * report$R0_annual, 1, sum)
@@ -540,12 +576,10 @@ RCM_posthoc_adjust <- function(report, obj, par = obj$env$last.par.best, dynamic
   lmid <- obj$env$data$lbinmid
   nlbin <- length(lmid)
   
-  report$CR <- report$Arec * report$EPR0 # Annual compensation ratio recalculated from annual EPR0
-  
   age_only_model <- data$len_age %>%
     apply(1, function(x) all(x == 1:data$n_age)) %>% 
     all()
-  if(age_only_model) {
+  if (age_only_model) {
     report$vul_len <- matrix(NA_real_, nlbin, data$nsel_block)
     report$ivul_len <- matrix(NA_real_, nlbin, dim(report$ivul)[3])
     
@@ -556,9 +590,9 @@ RCM_posthoc_adjust <- function(report, obj, par = obj$env$last.par.best, dynamic
     report$vul_len <- get_vul_len(report, data$vul_type, lmid, data$Linf)
     report$ivul_len <- get_ivul_len(report, data$ivul_type, lmid, data$Linf)
   }
-  if(dynamic_SSB0) report$dynamic_SSB0 <- RCM_dynamic_SSB0(obj, par)
+  if (dynamic_SSB0) report$dynamic_SSB0 <- RCM_dynamic_SSB0(obj, par)
   
-  if(data$comp_like == "mvlogistic") {
+  if (data$comp_like == "mvlogistic") {
     
     CAAmv <- calc_mvlogistic_loglike(obs = data$CAA_hist, pred = report$CAApred, 
                                      LWT = data$LWT_fleet[, 3], nllv = report$nll_fleet[1, , 3])
@@ -568,7 +602,7 @@ RCM_posthoc_adjust <- function(report, obj, par = obj$env$last.par.best, dynamic
     IAAmv <- calc_mvlogistic_loglike(obs = data$IAA_hist, pred = report$IAApred, 
                                      LWT = data$LWT_index[, 2], nllv = report$nll_index[1, , 2])
     
-    if(is.null(report$IALpred)) {
+    if (is.null(report$IALpred)) {
       IALmv <- list(tau = rep(NaN, data$nsurvey), nll = matrix(0, data$n_y, data$nsurvey))
     } else {
       IALmv <- calc_mvlogistic_loglike(obs = data$IAL_hist, pred = report$IALpred, 
@@ -581,8 +615,8 @@ RCM_posthoc_adjust <- function(report, obj, par = obj$env$last.par.best, dynamic
     report$nll_index[, 1:data$nsurvey, 2] <- IAAmv$nll
     report$nll_index[, 1:data$nsurvey, 3] <- IALmv$nll
     
-    report$compf <- cbind(CAAmv$tau, CALmv$tau)
-    report$compi <- cbind(IAAmv$tau, IALmv$tau)
+    report$log_compf <- log(cbind(CAAmv$tau, CALmv$tau))
+    report$log_compi <- log(cbind(IAAmv$tau, IALmv$tau))
   }
   return(report)
 }
@@ -607,7 +641,7 @@ get_vul_len <- function(report, selectivity, lmid, Linf) {
 get_ivul_len <- function(report, s_selectivity, lmid, Linf) {
   ivul_len <- matrix(NA_real_, length(lmid), length(s_selectivity)) # length-based: matrix of dimension nlbin, nsurvey
   for(i in 1:ncol(ivul_len)) {
-    if(s_selectivity[i] == -1 || s_selectivity[i] == 0) {
+    if (s_selectivity[i] == -1 || s_selectivity[i] == 0) {
       sls <- (report$iLFS[i] - report$iL5[i])/sqrt(-log(0.05, 2))
       srs <- (Linf - report$iLFS[i])/sqrt(-log(report$iVmaxlen[i], 2))
       
@@ -615,7 +649,7 @@ get_ivul_len <- function(report, s_selectivity, lmid, Linf) {
       dsc <- ifelse(report$iVmaxlen[i] > rep(0.99, length(lmid)), 1,
                     2^-((lmid - report$iLFS[i])/srs * (lmid - report$iLFS[i])/srs))
       ivul_len[, i] <- ifelse(lmid > report$iLFS[i], dsc, asc)
-    } else if(s_selectivity[i] > 0) {
+    } else if (s_selectivity[i] > 0) {
       ivul_len[, i] <- report$vul_len[, s_selectivity[i]]
     }
   }
@@ -626,7 +660,7 @@ RCM_SPR <- function(F_at_age, M, mat, wt, N_at_age, R, R_early, equilibrium = TR
   n_y <- nrow(F_at_age)
   n_age <- ncol(F_at_age)
   
-  if(equilibrium) { # Plusgroup always on
+  if (equilibrium) { # Plusgroup always on
     SSPR_F <- vapply(1:n_y, function(y) {
       yield_fn_SCA_int(max(F_at_age[y, ]), M = M[y, ], mat = mat[y, ], weight = wt[y, ], 
                        vul = F_at_age[y, ]/max(F_at_age[y, ]), Arec = 1, Brec = 1, opt = FALSE)["EPR"]
@@ -662,7 +696,7 @@ RCM_SPR <- function(F_at_age, M, mat, wt, N_at_age, R, R_early, equilibrium = TR
 
 dmvlogistic <- function(x, p, sd, xmin = 1e-8, log = FALSE) {
   resid <- log(x[x > xmin]) - log(p[x > xmin])
-  if(!length(resid)) stop("Density function can not be calculated from x")
+  if (!length(resid)) stop("Density function can not be calculated from x")
   accum <- log(sum(x[x <= xmin])) - log(sum(p[x <= xmin]))
   
   eta <- c(resid, accum) - mean(c(resid, accum))
@@ -671,7 +705,7 @@ dmvlogistic <- function(x, p, sd, xmin = 1e-8, log = FALSE) {
   
   # No normalizing constants!
   log_like <- -(A-1) * log(sd) - 0.5 * sum(eta^2)/sd/sd
-  if(log) {
+  if (log) {
     log_like
   } else {
     exp(log_like)
@@ -685,9 +719,9 @@ calc_mvlogistic_loglike <- function(obs, pred, nllv, LWT, obsmin = 1e-8) {
   
   vars <- lapply(1:nf, function(ff) {
     sum_count <- sapply(1:n_y, function(y) {
-      if(sum(obs[y, , ff]) > 0 && LWT[ff] > 0) {
+      if (sum(obs[y, , ff]) > 0 && LWT[ff] > 0) {
         A <- sum(obs[y, , ff] > obsmin)
-        if(any(obs[y, , ff] <= obsmin)) A <- A + 1
+        if (any(obs[y, , ff] <= obsmin)) A <- A + 1
         return(A-1)
       } else {
         return(0)
@@ -699,7 +733,7 @@ calc_mvlogistic_loglike <- function(obs, pred, nllv, LWT, obsmin = 1e-8) {
     tau <- sqrt(tau2)
     
     log_like <- sapply(1:n_y, function(y) {
-      if(sum(obs[y, , ff]) > 0 && LWT[ff] > 0) {
+      if (sum(obs[y, , ff]) > 0 && LWT[ff] > 0) {
         log_like <- dmvlogistic(x = obs[y, , ff], p = pred[y, , ff]/sum(pred[y, , ff]), 
                                 sd = tau, xmin = obsmin, log = TRUE)
       } else {
