@@ -79,7 +79,7 @@ compare_models <- function(..., label = NULL, color = NULL) {
   if (!all(is.na(R))) ts_matplot(R, "Recruitment", color = color)
 
   par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
-  plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
+  graphics::plot.default(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
   legend("bottom", label, col = color, xpd = TRUE, horiz = TRUE, bty = "n", lwd = 2)
 
   invisible()
@@ -700,6 +700,7 @@ plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, plot_type = c(
         points(Year[i], data_val[j], cex = 0.5 * diameter_max * pmin(obs[i, j], n2), pch = 21, bg = "white")
       }
     }
+    lapply(pretty(c(min(Year) - max(data_val), max(Year))), function(x) abline(a = -x, b = 1, col = "gray50", lty = "dotted"))
     legend("topleft", legend = c(n1, paste0(">", n2)), pt.cex = 0.5 * diameter_max * c(n1, n2),
            pt.bg = "white", pch = 21, horiz = TRUE)
     return(invisible())
@@ -720,6 +721,9 @@ plot_composition <- function(Year = 1:nrow(obs), obs, fit = NULL, plot_type = c(
     isPositive <- resid > 0
     points(Year_mat[!isPositive], data_mat[!isPositive], cex = pmin(0.5 * diameter_max * abs(resid[!isPositive]), diameter_max), pch = 21, bg = bubble_color[1])
     points(Year_mat[isPositive], data_mat[isPositive], cex = pmin(0.5 * diameter_max * resid[isPositive], diameter_max), pch = 21, bg = bubble_color[2])
+    
+    lapply(pretty(c(min(Year) - max(data_val), max(Year))), function(x) abline(a = -x, b = 1, col = "gray50", lty = "dotted"))
+    
     legend("topleft", legend = c("<-10", "-1", "1", ">10"),
            pt.cex = c(diameter_max, 0.5 * diameter_max, 0.5 * diameter_max, diameter_max),
            pt.bg = rep(bubble_color, each = 2), pch = 21, horiz = TRUE)
@@ -862,22 +866,31 @@ plot_Kobe <- function(biomass, exploit, arrow_size = 0.07, color = TRUE, xlab =
 #' @param ylab Character string for label on y-axis.
 #' @author Q. Huynh
 #' @return A stock-recruit plot
+#' @importFrom graphics text
 #' @export plot_SR
 plot_SR <- function(Spawners, expectedR, R0 = NULL, S0 = NULL, rec_dev = NULL, trajectory = FALSE,
                     y_zoom = NULL, ylab = "Recruitment") {
   if (is.null(rec_dev)) {
-    R.max <- 1.1 * max(expectedR)
+    R.max <- max(expectedR)
   } else {
-    if (is.null(y_zoom)) R.max <- 1.1 * max(rec_dev)
+    if (is.null(y_zoom)) R.max <- max(rec_dev)
     else R.max <- y_zoom * max(expectedR)
   }
-  S.max <- 1.1 * max(c(Spawners, S0))
-  plot(Spawners[order(Spawners)], expectedR[order(Spawners)], typ = "l", xlim = c(0, 1.05 * S.max), ylim = c(0, 1.1 * R.max),
+  S.max <- max(c(Spawners, S0))
+  plot(Spawners[order(Spawners)], expectedR[order(Spawners)], typ = "l", xlim = c(0, 1.1 * S.max), ylim = c(0, 1.1 * R.max),
        xlab = "Spawning Stock Biomass (SSB)", lwd = ifelse(is.null(rec_dev), 1, 3), ylab = ylab)
   if (trajectory) {
     n.arrows <- length(Spawners)
     arrows(x0 = Spawners[1:(n.arrows-1)], y0 = rec_dev[1:(n.arrows-1)],
            x1 = Spawners[2:n.arrows], y1 = rec_dev[2:n.arrows], length = 0.07)
+    
+    if (any(expectedR != rec_dev)) {
+      txt_ind <- seq(1, length(rec_dev), length.out = 10)
+      if (all(txt_ind != length(rec_dev))) txt_ind <- c(txt_ind, length(rec_dev))
+      text(Spawners[txt_ind], rec_dev[txt_ind], labels = names(rec_dev)[txt_ind], pos = 3)
+      text(Spawners[txt_ind], rec_dev[txt_ind], labels = names(rec_dev)[txt_ind], pos = 3)
+    }
+    
   } else {
     if (is.null(rec_dev)) {
       points(Spawners, expectedR)
